"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const compression_1 = require("./compression");
const enums_1 = require("./enums");
const fast_url_parser_1 = __importDefault(require("fast-url-parser"));
const querystring_1 = require("querystring");
class Http {
    constructor() {
        this.httpAgent = new http_1.default.Agent({
            keepAlive: true,
            maxSockets: 50,
            maxFreeSockets: 30,
            keepAliveMsecs: 30000,
        });
        this.httpsAgent = new https_1.default.Agent({
            keepAlive: true,
            maxSockets: 50,
            maxFreeSockets: 30,
            keepAliveMsecs: 30000,
        });
    }
    request(url, requestOptions, cb) {
        const requestProvider = url.startsWith('https') ? {
            agent: this.httpsAgent,
            client: https_1.default
        } : {
            agent: this.httpAgent,
            client: http_1.default
        };
        const requestBody = typeof requestOptions.body === "object" ?
            JSON.stringify(requestOptions.body) :
            typeof requestOptions.body === "string" ?
                requestOptions.body : undefined;
        const requestFormContent = !requestBody ?
            typeof requestOptions.form === "object" ?
                querystring_1.stringify(requestOptions.form) :
                typeof requestOptions.form === "string" ?
                    requestOptions.form : undefined : undefined;
        const options = this.createRequestOptions(url, requestOptions, requestProvider.agent, requestBody, requestFormContent);
        const request = requestProvider.client.request(options, response => {
            compression_1.Compression
                .handle(response, (err, body) => {
                if (err)
                    return cb(err);
                cb(null, {
                    body,
                    response
                });
            });
        });
        request.setNoDelay(true);
        if (options.timeout) {
            request.setTimeout(options.timeout, () => {
                request.abort();
            });
        }
        request
            .on('error', e => {
            cb(e);
        });
        const writableContent = requestBody || requestFormContent;
        if (writableContent) {
            if (requestOptions.compressRequest) {
                compression_1.Compression.compressBody(writableContent, (err, buffer) => {
                    if (err || !buffer) {
                        request.setHeader('content-length', Buffer.byteLength(writableContent));
                        request.write(writableContent);
                    }
                    else {
                        request.setHeader('content-encoding', 'gzip');
                        request.setHeader('content-length', Buffer.byteLength(buffer));
                        request.write(buffer);
                    }
                    request.end();
                });
            }
            else {
                request.setHeader('content-length', Buffer.byteLength(writableContent));
                request.write(writableContent);
                request.end();
            }
        }
        else {
            request.end();
        }
    }
    createRequestOptions(targetUrl, options, agent, bodyContent, formContent) {
        const url = fast_url_parser_1.default.parse(targetUrl);
        const mergedOptions = {
            method: options.method || 'get',
            agent,
            hostname: url.hostname,
            port: url.port,
            protocol: url._protocol + ':',
            path: url.pathname + (url.search || ''),
            headers: Object.assign(Object.assign({}, options.headers), { 'accept-encoding': compression_1.Compression.getSupportedStreams() })
        };
        if (typeof options.followRedirect === "boolean") {
            mergedOptions.followRedirect = options.followRedirect;
        }
        if (options.httpTimeout) {
            mergedOptions.timeout = options.httpTimeout;
        }
        if (options.json) {
            mergedOptions.headers['content-type'] = enums_1.CONTENT_TYPE.ApplicationJson;
        }
        if (bodyContent) {
            mergedOptions.headers['content-type'] = enums_1.CONTENT_TYPE.ApplicationJson;
        }
        else if (formContent) {
            mergedOptions.headers['content-type'] = enums_1.CONTENT_TYPE.FormUrlEncoded;
        }
        return mergedOptions;
    }
}
exports.Http = Http;
//# sourceMappingURL=http.js.map